//
//  main.cpp
//  07-GLSLTextures
//
//  Created by Orlando Pereira on 20/05/15.
//  Copyright (c) 2015 Orlando Pereira. All rights reserved.
//

#include <GLUT/glut.h>
#include <iostream>

#include "textfile.h"
#include "SOIL.h"

GLuint v,f,f2,p;

GLuint tex_ID, tex_ID2;

int screenWidth = 1024;
int screenHeight = 1024;
bool mouseLeftDown;
bool mouseRightDown;
float mouseX, mouseY, cameraAngleX, cameraAngleY;
float cameraDistance = 5.0f;

static void render() {
    
    glClearColor(0.0, 0.0, 0.0, 1.0);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glPushMatrix();
    
    glEnable(GL_DEPTH_TEST);
    
    glTranslatef(0, 0, -cameraDistance);
    glRotatef(cameraAngleX, 1, 0, 0);   // pitch
    glRotatef(cameraAngleY, 0, 1, 0);   // heading
    
    glBegin(GL_QUADS);
    glNormal3f( -1, -1, 1.0f );
    glVertex3f( -2.0f, -2.0f, -0.1f );
    
    glNormal3f( 1, -1, 1.0f );
    glVertex3f( 2.0f, -2.0f, -0.1f );
    
    glNormal3f( 1, 1, 1.0f );
    glVertex3f( 2.0f, 2.0f, -0.1f );
    
    glNormal3f( -1, 1, 1.0f );
    glVertex3f( -2.0f, 2.0f, -0.1f );
    glEnd();
    glPopMatrix();
    
    glutSwapBuffers();
}

void printShaderInfoLog(GLuint obj) {
    int infologLength = 0;
    int charsWritten  = 0;
    char *infoLog;
    
    glGetShaderiv(obj, GL_INFO_LOG_LENGTH,&infologLength);
    
    if (infologLength > 0) {
        infoLog = (char *)malloc(infologLength);
        glGetShaderInfoLog(obj, infologLength, &charsWritten, infoLog);
        printf("%s\n",infoLog);
        free(infoLog);
    }
}

void printProgramInfoLog(GLuint obj) {
    int infologLength = 0;
    int charsWritten  = 0;
    char *infoLog;
    
    glGetProgramiv(obj, GL_INFO_LOG_LENGTH,&infologLength);
    
    if (infologLength > 0) {
        infoLog = (char *)malloc(infologLength);
        glGetProgramInfoLog(obj, infologLength, &charsWritten, infoLog);
        printf("%s\n",infoLog);
        free(infoLog);
    }
}

void setShaders() {
    
    char *vs = NULL, *fs = NULL;
    
    v = glCreateShader(GL_VERTEX_SHADER);
    f = glCreateShader(GL_FRAGMENT_SHADER);
    f2 = glCreateShader(GL_FRAGMENT_SHADER);
    
    vs = textFileRead("minimal.vert");
    fs = textFileRead("minimal.frag");
    
    const char * vv = vs;
    const char * ff = fs;
    
    glShaderSource(v, 1, &vv,NULL);
    glShaderSource(f, 1, &ff,NULL);
    
    free(vs);free(fs);
    
    glCompileShader(v);
    glCompileShader(f);
    
    printShaderInfoLog(v);
    printShaderInfoLog(f);
    printShaderInfoLog(f2);
    
    p = glCreateProgram();
    glAttachShader(p,v);
    glAttachShader(p,f);
    
    glLinkProgram(p);
    printProgramInfoLog(p);
    
    glUseProgram(p);
}

void idleCB() {
    glutPostRedisplay();
}

void reshapeCB(int w, int h) {
    screenWidth = w;
    screenHeight = h;
    
    glViewport(0, 0, (GLsizei)screenWidth, (GLsizei)screenHeight);
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0f, (float)(screenWidth)/screenHeight, 1.0f, 1000.0f);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void mouseCB(int button, int state, int x, int y) {
    mouseX = x;
    mouseY = y;
    
    if(button == GLUT_LEFT_BUTTON) {
        if(state == GLUT_DOWN)
            mouseLeftDown = true;
        else if(state == GLUT_UP)
            mouseLeftDown = false;
    } else if(button == GLUT_RIGHT_BUTTON) {
        if(state == GLUT_DOWN)
            mouseRightDown = true;
        else if(state == GLUT_UP)
            mouseRightDown = false;
    }
}

void mouseMotionCB(int x, int y) {
    if(mouseLeftDown) {
        cameraAngleY += (x - mouseX);
        cameraAngleX += (y - mouseY);
        mouseX = x;
        mouseY = y;
    }
    if(mouseRightDown) {
        cameraDistance -= (y - mouseY) * 0.2f;
        mouseY = y;
    }
}

static void keyboard(unsigned char key, int x, int y) {
    switch (key) {
        case 27:
            exit(0);
    }
}

int main(int argc, char **argv) {
    
    glutInit(&argc, argv);
    glutInitWindowSize(screenWidth, screenHeight);
    glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_DEPTH);
    glutCreateWindow("GLSL Textures");
    
    glutKeyboardFunc(keyboard);
    glutDisplayFunc(render);
    glutMouseFunc(mouseCB);
    glutMotionFunc(mouseMotionCB);
    glutIdleFunc(idleCB);
    glutReshapeFunc(reshapeCB);
    
    // ----- LOAD TEXTURE CODE USING THE EXTERNAL SOIL LIB -----
    
    std::string load_me;
    if( load_me.length() > 2 ) {
        load_me = load_me.substr( 0, load_me.length() - 0 );
    } else {
//        load_me = "field_128_cube.dds";
//        load_me = "img_cheryl.jpg";
//        load_me = "img_test_indexed.tga";
//        load_me = "img_test.bmp";
//        load_me = "img_test.dds";
//        load_me = "img_test.tga";
        load_me = "img_test.png";
//        load_me = "test_rect.png";
    }
    std::cout << "'" << load_me << "'" << std::endl;
    
    std::cout << "Attempting to load as a cubemap" << std::endl;
    tex_ID = SOIL_load_OGL_single_cubemap(
                                          load_me.c_str(),
                                          SOIL_DDS_CUBEMAP_FACE_ORDER,
                                          SOIL_LOAD_AUTO,
                                          SOIL_CREATE_NEW_ID,
                                          SOIL_FLAG_POWER_OF_TWO
                                          | SOIL_FLAG_MIPMAPS
                                          | SOIL_FLAG_DDS_LOAD_DIRECT
                                          );
    
    if( tex_ID > 0 ) {
        glEnable( GL_TEXTURE_CUBE_MAP );
        glEnable( GL_TEXTURE_GEN_S );
        glEnable( GL_TEXTURE_GEN_T );
        glEnable( GL_TEXTURE_GEN_R );
        glTexGeni( GL_S, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP );
        glTexGeni( GL_T, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP );
        glTexGeni( GL_R, GL_TEXTURE_GEN_MODE, GL_REFLECTION_MAP );
        glBindTexture( GL_TEXTURE_CUBE_MAP, tex_ID );
        std::cout << "the loaded single cube map ID was " << tex_ID << std::endl;
    } else {
        std::cout << "Attempting to load as a HDR texture" << std::endl;
        tex_ID = SOIL_load_OGL_HDR_texture(
                                           load_me.c_str(),
                                           SOIL_HDR_RGBdivA2,
                                           0,
                                           SOIL_CREATE_NEW_ID,
                                           SOIL_FLAG_POWER_OF_TWO
                                           | SOIL_FLAG_MIPMAPS
                                           );
        if( tex_ID < 1 )  {
            //	loading of the single-image-cubemap failed, try it as a simple texture
            std::cout << "Attempting to load as a simple 2D texture" << std::endl;
            //	load the texture, if specified
            tex_ID = SOIL_load_OGL_texture(
                                           load_me.c_str(),
                                           SOIL_LOAD_AUTO,
                                           SOIL_CREATE_NEW_ID,
                                           SOIL_FLAG_POWER_OF_TWO
                                           | SOIL_FLAG_MIPMAPS
                                           | SOIL_FLAG_DDS_LOAD_DIRECT
                                           );
        }
        
        if( tex_ID > 0 ) {
            //glEnable(GL_TEXTURE_2D);
            glBindTexture( GL_TEXTURE_2D, tex_ID );
            std::cout << "the loaded texture ID was " << tex_ID << std::endl;
        } else {
            glDisable(GL_TEXTURE_2D);
            std::cout << "Texture loading failed: '" << SOIL_last_result() << "'" << std::endl;
        }
    }
    
    glutMainLoop();
    return 0;
}
